//********************************************************************************
//********************************************************************************
//** Version 1.4																**
//** Email : arm@ns.ph.liv.ac.uk (Andrew Mather)								**
//**																			**
//** Version 1.4 Completed 8th November 2004									**
//**																			**
//**																			**
//** Requires installation of Java Advanced Imaging API (JAI)					**
//** 	(http://java.sun.com/products/java-media/jai/)							**
//**																			**
//********************************************************************************
//********************************************************************************

package sinogram;

import java.io.*;
import javax.imageio.*;
import java.awt.image.BufferedImage;
import ellipse.*;

public class generateSinogram1
{
	protected double [][] sinogram;
	protected int numOfProjections;
	protected int numOfPositions;

	private ellipseList ellipseList = new ellipseList();

	//********************************************************************************
	//************************* Constructor(s) ***************************************
	//********************************************************************************

	public generateSinogram1(String pathEllipseFile, int projections , int positions) throws IOException
	{
		ellipseList.loadList(pathEllipseFile);

		numOfProjections	= projections;
		numOfPositions		= positions;

		sinogram = new double[projections][positions];
	}

	//********************************************************************************
	//************************ Instance Method(s) ************************************
	//********************************************************************************

	public void calculateSinogram(double sigma)
	{
		//*************************************************************
		//*** Calculate the Sinogram from the ellipse list ************
		//*************************************************************

		double numOfProjections = sinogram.length;
		double numOfPositions	= sinogram[0].length;
		double sizeOfElement	= 1.0 / numOfProjections;

		for(int i = 0 ; i < numOfProjections ; i++)
		{
			double angle = ( i / (double)numOfProjections ) * Math.PI;

			for(int e = 0 , n = ellipseList.size()  ; e < n ; e++)
				((ellipse)ellipseList.get(e)).setTheta(angle);

			for(int j = 0 ; j < numOfPositions ; j++)
			{
				double t = ( (j) / ((double)numOfPositions - 1) ) * 2.0  - 1.0;

				for(int e = 0 , n = ellipseList.size()  ; e < n ; e++)
				{
					for( double k = -1.0 ; k <= 1.0 ; k += 0.02 )
						sinogram[i][j] += ((ellipse)ellipseList.get(e)).getP(t + k*sizeOfElement);

				}

				if(sigma > 0 )	sinogram[i][j] = gaussianSpread(sinogram[i][j], sigma);
				if(sigma < 0 )	sinogram[i][j] = gaussianSpread(sinogram[i][j], -sigma * sinogram[i][j]);
			}
		}
	}

	//****************************************************************************

	public void saveSinogramData(String strOuputFile) throws IOException
	{
		//*************************************************************
		//*** Save the Sinogram to a .dat file ************************
		//*************************************************************

		FileWriter file 		= new FileWriter (strOuputFile);
		PrintWriter outputFile 	= new PrintWriter(file);

		for(int j = 0 ; j < sinogram.length ; j++)
		{
			for(int i = 0 ; i < sinogram[0].length ; i++)
			{
				double value = sinogram[j][i];
				outputFile.print(value + " ");
			}
			outputFile.print("\n");
		}
		outputFile.close();
	}

	//****************************************************************************

	public void saveSinogramImage(String strOuputFile) throws IOException
	{
		//*************************************************************
		//*** Save the Sinogram to   a .png file **********************
		//*************************************************************
		double min  = 1e9;
		double max  = 0;

		for(int j = 0 ; j < sinogram[0].length ; j++)
			for(int i = 0 ; i < sinogram.length ; i++)
			{
				if( sinogram[i][j] < min ) min = sinogram[i][j];
				if( sinogram[i][j] > max ) max = sinogram[i][j];
			}

		double scaleValue	= (max - min) / 255.0;

		File file 				= new File(strOuputFile);
		BufferedImage imagePNG 	= new BufferedImage( sinogram.length , sinogram[0].length , BufferedImage.TYPE_3BYTE_BGR);

		for(int j = 0 ; j < sinogram.length ; j++)
			for(int i = 0 ; i < sinogram[0].length ; i++)
			{
				int value = (int)(((sinogram[j][i] - min) / scaleValue) + 0.5);
				imagePNG.setRGB(j, i, value * (65536 + 256 + 1));
			}

		ImageIO.write(imagePNG, "png", file );
	}

	//********************************************************************************

	public void saveSourceDist(int size , String path) throws IOException
	{
		//********************************************************************
		//*** Save the source distribution (from the ellipse data) to .png ***
		//********************************************************************

		BufferedImage sourcePNG;
		File outputFile;

		double [][] source	 = new double[size][size];
		double min 			 = 1e9;
		double max 	  		 = 0;
		double scaleValue 	 = 0;

		for(int i = 0 ; i < size ; i++)
		{
			for(int j = 0 ; j < size ; j++)
			{
				double x = ( j / (double)size ) * 2 - 1;
				double y = -(( i / (double)size ) * 2 - 1);

				for(int e = 0 ; e < ellipseList.size() ; e++)
					source[j][i] += ((ellipse)ellipseList.get(e)).isPointIn(x , y);

				if( source[j][i] < min ) min = source[j][i];
				if( source[j][i] > max ) max = source[j][i];
			}
		}

		sourcePNG = new BufferedImage(size , size , BufferedImage.TYPE_3BYTE_BGR );
		scaleValue = (max - min) / 255.0;

		for(int i = 0 ; i < size ; i++)
		{
			for(int j = 0 ; j < size ; j++)
			{
				int value = (int)(((source[i][j] - min) / scaleValue) + 0.5);
				sourcePNG.setRGB(i, j, value * (65536 + 256 + 1));
			}
		}

		outputFile = new File(path);
		ImageIO.write(sourcePNG, "png", outputFile );
	}

	//****************************************************************************

	private double gaussianSpread(double mean, double sigma)
	{
		double sum = 0;

		for(int i = 0 ; i < 12 ; i++) sum += Math.random();

		return mean + sigma * ( sum - 6.0);
	}
}