//********************************************************************************
//********************************************************************************
//** Version 1.1																**
//** Email : arm@ns.ph.liv.ac.uk (Andrew Mather)								**
//**																			**
//** Version 1.1 Completed 9th November 2004									**
//**																			**
//**																			**
//** Requires installation of Java Advanced Imaging API (JAI)					**
//** 	(http://java.sun.com/products/java-media/jai/)							**
//**																			**
//********************************************************************************
//********************************************************************************

package sinogram;

import java.io.*;
import javax.imageio.*;
import java.awt.image.BufferedImage;

public class generateSinogram2
{
	protected final double TO_RADIANS = Math.PI / 180.0;
	protected double [][] sinogram;

	protected int numOfProjections;
	protected int numOfPositions;
	protected int image_size;

	private BufferedImage source_image;

	//********************************************************************************
	//************************* Constructor(s) ***************************************
	//********************************************************************************

	public generateSinogram2(String pathImageFile, int projections , int positions) throws IOException
	{
		source_image = ImageIO.read(new File(pathImageFile));					//*** Loads the file input an image format "BufferedImage"

		numOfProjections	= projections;
		numOfPositions		= positions;

		sinogram = new double[projections][positions];

		image_size	= source_image.getWidth();									//*** Number of pixels for length and width of image
	}

	//********************************************************************************
	//************************ Instance Method(s) ************************************
	//********************************************************************************

	public void calculateSinogram(double sigma)
	{
		//*************************************************************
		//*** Calculate the Sinogram from the imae file ***************
		//*************************************************************

		double [][] lookupTable = new double[numOfProjections][2];		//*** Sin / Cos lookup array (saves processing time)

		double xp , yp , pos , sum;
		int i , j , k , p , bin;
		int pixels 			= image_size * image_size;

		double deltaTheta  	= Math.PI / (double)numOfProjections;
		double deltaR		= (double)numOfPositions / (double)image_size;
		double delta1overR  = (1 / deltaR) * 0.5;

		double mid_x 		= deltaR * image_size / 2.0 - deltaR * 0.5;
		double mid_y 		= mid_x;

		double area 		= Math.PI;
		double thisBin , nextBin , remainder , theta , d , pixelValue;

		//************************************************************
		//**** Step 1 : Forward Project ******************************
		//************************************************************

		for( i = 0 ; i < numOfProjections ; i++)
		{
			lookupTable[i][0] = Math.cos( i * deltaTheta );
			lookupTable[i][1] = Math.sin( i * deltaTheta );
		}

		for( k = 0 ; k < numOfProjections ; k++)
		{
			for( p = 0 ; p < pixels ; p++)
			{
				i = p % image_size;
				j = p / image_size;

				xp  = (i * deltaR) - mid_x;
				yp  = (-j* deltaR) + mid_y;

				pos = xp * lookupTable[k][0] + yp * lookupTable[k][1] + numOfPositions / 2.0 + 0.5 * deltaR;
				bin = (int)pos;

				if(pos >= 0 && bin < numOfPositions)
				{
					remainder 	= pos - bin;
					d			= 1.0 - Math.abs( 1.0 - 2 * remainder);
					nextBin 	= Math.acos(d) - d * Math.sqrt( 1 - d*d);
					thisBin		= area - nextBin;
					pixelValue  = source_image.getRGB(i,j)&0x000000ff;

					if(sigma != 0)
					{
						pixelValue = gaussianSpread(pixelValue , sigma * Math.sqrt(pixelValue));
					}

					if(remainder < delta1overR)
					{
						if(bin >= 1)
						{
							sinogram[k][bin - 1] += pixelValue * nextBin;
						}
						sinogram[k][bin] += pixelValue * thisBin;
					}
					else
					{
						if(bin < numOfPositions - 1)
						{
							sinogram[k][bin + 1] += pixelValue * nextBin;
						}
						sinogram[k][bin] += pixelValue * thisBin;
					}
				}
			}
		}
	}

	//****************************************************************************

	public void saveSinogramData(String strOuputFile) throws IOException
	{
		//*************************************************************
		//*** Save the Sinogram to a .dat file ************************
		//*************************************************************

		FileWriter file 		= new FileWriter (strOuputFile);
		PrintWriter outputFile 	= new PrintWriter(file);

		for(int j = 0 ; j < sinogram.length ; j++)
		{
			for(int i = 0 ; i < sinogram[0].length ; i++)
			{
				double value = sinogram[j][i];
				outputFile.print(value + " ");
			}
			outputFile.print("\n");
		}
		outputFile.close();
	}

	//****************************************************************************

	public void saveSinogramImage(String strOuputFile) throws IOException
	{
		//*************************************************************
		//*** Save the Sinogram to   a .png file **********************
		//*************************************************************
		double min  = 1e9;
		double max  = 0;

		for(int j = 0 ; j < sinogram[0].length ; j++)
			for(int i = 0 ; i < sinogram.length ; i++)
			{
				if( sinogram[i][j] < min ) min = sinogram[i][j];
				if( sinogram[i][j] > max ) max = sinogram[i][j];
			}

		double scaleValue	= (max - min) / 255.0;

		File file 				= new File(strOuputFile);
		BufferedImage imagePNG 	= new BufferedImage( sinogram.length , sinogram[0].length , BufferedImage.TYPE_3BYTE_BGR);

		for(int j = 0 ; j < sinogram.length ; j++)
			for(int i = 0 ; i < sinogram[0].length ; i++)
			{
				int value = (int)(((sinogram[j][i] - min) / scaleValue) + 0.5);
				imagePNG.setRGB(j, i, value * (65536 + 256 + 1));
			}

		ImageIO.write(imagePNG, "png", file );
	}

	//****************************************************************************

	private double gaussianSpread(double mean, double sigma)
	{
		double sum = 0;

		for(int i = 0 ; i < 12 ; i++) sum += Math.random();

		return mean + sigma * ( sum - 6.0);
	}

	//******************************************************************************************
}